@@ -102,6 +102,14 @@ abstract class KyuubiSparkSQLParserBase extends ParserInterface with SQLConfHelp
   override def parseDataType(sqlText: String): DataType = {
     delegate.parseDataType(sqlText)
   }
+
+  /**
+   * This functions was introduced since spark-3.3, for more details, please see
+   * https://github.com/apache/spark/pull/34543
+   */
+  override def parseQuery(sqlText: String): LogicalPlan = {
+    delegate.parseQuery(sqlText)
+  }
 }
 
 class SparkKyuubiSparkSQLParser(
