@@ -16,12 +16,13 @@
  */
 package org.apache.kyuubi.plugin.spark.authz.ranger
 
+import scala.util.Properties
+
 import org.scalatest.Outcome
 
 import org.apache.kyuubi.Utils
 import org.apache.kyuubi.plugin.spark.authz.AccessControlException
 import org.apache.kyuubi.plugin.spark.authz.RangerTestUsers._
-import org.apache.kyuubi.plugin.spark.authz.util.AuthZUtils._
 import org.apache.kyuubi.tags.PaimonTest
 import org.apache.kyuubi.util.AssertionUtils._
 
@@ -31,8 +32,9 @@ import org.apache.kyuubi.util.AssertionUtils._
 @PaimonTest
 class PaimonCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   override protected val catalogImpl: String = "hive"
-  private def isSupportedVersion = isScalaV212
+  private def isSupportedVersion = true
 
+  val scalaVersion: String = Properties.versionString
   val catalogV2 = "paimon_catalog"
   val namespace1 = "paimon_ns"
   val table1 = "table1"
@@ -49,8 +51,9 @@ class PaimonCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
         s"spark.sql.catalog.$catalogV2.warehouse",
         Utils.createTempDir(catalogV2).toString)
       super.beforeAll()
-      doAs(admin, sql(s"CREATE DATABASE IF NOT EXISTS $catalogV2.$namespace1"))
     }
+
+    doAs(admin, sql(s"CREATE DATABASE IF NOT EXISTS $catalogV2.$namespace1"))
   }
 
   override def afterAll(): Unit = {
@@ -84,20 +87,28 @@ class PaimonCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
 
   test("[KYUUBI #6541] INSERT/SELECT TABLE") {
     val tName = "t_paimon"
-    withCleanTmpResources(Seq((s"$catalogV2.$namespace1.$tName", "table"))) {
-      doAs(bob, sql(createTableSql(namespace1, tName)))
 
-      interceptEndsWith[AccessControlException] {
-        doAs(someone, sql(s"INSERT INTO $catalogV2.$namespace1.$tName VALUES (1, 'name_1')"))
-      }(s"does not have [update] privilege on [$namespace1/$tName]")
-      doAs(bob, sql(s"INSERT INTO $catalogV2.$namespace1.$tName VALUES (1, 'name_1')"))
-      doAs(bob, sql(s"INSERT INTO $catalogV2.$namespace1.$tName VALUES (1, 'name_2')"))
+    /**
+     * paimon-spark run on Scala 2.12.
+     */
+    if (scalaVersion.startsWith("version 2.12")) {
+      withCleanTmpResources(Seq((s"$catalogV2.$namespace1.$tName", "table"))) {
 
-      interceptEndsWith[AccessControlException] {
-        doAs(someone, sql(s"SELECT id FROM $catalogV2.$namespace1.$tName").show())
-      }(s"does not have [select] privilege on [$namespace1/$tName/id]")
-      doAs(bob, sql(s"SELECT name FROM $catalogV2.$namespace1.$tName").show())
+        doAs(bob, sql(createTableSql(namespace1, tName)))
+
+        interceptEndsWith[AccessControlException] {
+          doAs(someone, sql(s"INSERT INTO $catalogV2.$namespace1.$tName VALUES (1, 'name_1')"))
+        }(s"does not have [update] privilege on [$namespace1/$tName]")
+        doAs(bob, sql(s"INSERT INTO $catalogV2.$namespace1.$tName VALUES (1, 'name_1')"))
+        doAs(bob, sql(s"INSERT INTO $catalogV2.$namespace1.$tName VALUES (1, 'name_2')"))
+
+        interceptEndsWith[AccessControlException] {
+          doAs(someone, sql(s"SELECT id FROM $catalogV2.$namespace1.$tName").show())
+        }(s"does not have [select] privilege on [$namespace1/$tName/id]")
+        doAs(bob, sql(s"SELECT name FROM $catalogV2.$namespace1.$tName").show())
+      }
     }
+
   }
 
   def createTableSql(namespace: String, table: String): String =
