@@ -74,7 +74,6 @@ class ExecuteStatement(
           val ke =
             KyuubiSQLException("Error submitting query in background, query rejected", rejected)
           setOperationException(ke)
-          shutdownTimeoutMonitor()
           throw ke
       }
     } else {
@@ -109,10 +108,13 @@ class ExecuteStatement(
   private def executeStatement(trinoStatement: TrinoStatement): Unit = {
     setState(OperationState.RUNNING)
     try {
+      trinoStatement.printStatusInfo()
       schema = trinoStatement.getColumns
       val resultSet = trinoStatement.execute()
       iter =
         if (incrementalCollect) {
+          val hasResult = resultSet.hasNext
+          debug(s"Increment collect mode hasResult: $hasResult")
           info("Execute in incremental collect mode")
           FetchIterator.fromIterator(resultSet)
         } else {
@@ -125,6 +127,7 @@ class ExecuteStatement(
     } catch {
       onError(cancel = true)
     } finally {
+      trinoStatement.stopPrinter()
       shutdownTimeoutMonitor()
     }
   }
