@@ -74,6 +74,16 @@ abstract class AbstractOperation(session: Session) extends Operation with Loggin
 
   override def getOperationLog: Option[OperationLog] = None
 
+  override def withOperationLog(f: => Unit): Unit = {
+    try {
+      getOperationLog.foreach(OperationLog.setCurrentOperationLog)
+      f
+    } finally {
+      OperationLog.removeCurrentOperationLog()
+    }
+  }
+
+  OperationAuditLogger.audit(this, OperationState.INITIALIZED)
   @volatile protected var state: OperationState = INITIALIZED
   @volatile protected var startTime: Long = _
   @volatile protected var completedTime: Long = _
@@ -103,7 +113,6 @@ abstract class AbstractOperation(session: Session) extends Operation with Loggin
 
   protected def setOperationException(opEx: KyuubiSQLException): Unit = {
     this.operationException = opEx
-    error(s"Error operating $opType: ${opEx.getMessage}", opEx)
   }
 
   def getOperationJobProgress: TProgressUpdateResp = operationJobProgress
@@ -127,6 +136,7 @@ abstract class AbstractOperation(session: Session) extends Operation with Loggin
     }
     state = newState
     lastAccessTime = System.currentTimeMillis()
+    OperationAuditLogger.audit(this, state)
   }
 
   protected def isClosedOrCanceled: Boolean = {
