@@ -17,15 +17,12 @@
 
 package org.apache.kyuubi.util
 
-import java.nio.ByteBuffer
 import java.time.{Instant, LocalDate, LocalDateTime, LocalTime, ZoneId}
 import java.time.chrono.IsoChronology
 import java.time.format.DateTimeFormatterBuilder
 import java.time.temporal.ChronoField
 import java.util.{Date, Locale}
 
-import scala.language.implicitConversions
-
 import org.apache.commons.lang3.time.FastDateFormat
 
 private[kyuubi] object RowSetUtils {
@@ -77,8 +74,4 @@ private[kyuubi] object RowSetUtils {
     timeZone.map(timestampFormatter.withZone(_).format(i))
       .getOrElse(timestampFormatter.format(i))
   }
-
-  implicit def bitSetToBuffer(bitSet: java.util.BitSet): ByteBuffer = {
-    ByteBuffer.wrap(bitSet.toByteArray)
-  }
 }
