@@ -25,4 +25,9 @@ class AnonymousAuthenticationProviderImplSuite extends KyuubiFunSuite {
     new AnonymousAuthenticationProviderImpl().authenticate("test", "test")
   }
 
+  test("testAuthenticateToken") {
+    new AnonymousAuthenticationProviderImpl()
+      .authenticate(DefaultTokenCredential("test", Map.empty))
+  }
+
 }
