@@ -24,7 +24,7 @@ import scala.collection.JavaConverters._
 import scala.util.control.NonFatal
 
 import com.google.common.cache.{Cache, CacheBuilder, RemovalNotification}
-import io.fabric8.kubernetes.api.model.{ContainerState, Pod}
+import io.fabric8.kubernetes.api.model.{ContainerState, Pod, Service}
 import io.fabric8.kubernetes.client.KubernetesClient
 import io.fabric8.kubernetes.client.informers.{ResourceEventHandler, SharedIndexInformer}
 
@@ -36,7 +36,8 @@ import org.apache.kyuubi.config.KyuubiConf.KubernetesApplicationStateSource.Kube
 import org.apache.kyuubi.config.KyuubiConf.KubernetesCleanupDriverPodStrategy.{ALL, COMPLETED, NONE}
 import org.apache.kyuubi.engine.ApplicationState.{isTerminated, ApplicationState, FAILED, FINISHED, KILLED, NOT_FOUND, PENDING, RUNNING, UNKNOWN}
 import org.apache.kyuubi.operation.OperationState
-import org.apache.kyuubi.server.metadata.MetadataManager
+import org.apache.kyuubi.server.KyuubiServer
+import org.apache.kyuubi.session.KyuubiSessionManager
 import org.apache.kyuubi.util.{KubernetesUtils, ThreadUtils}
 
 class KubernetesApplicationOperation extends ApplicationOperation with Logging {
@@ -46,6 +47,8 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
     new ConcurrentHashMap[KubernetesInfo, KubernetesClient]
   private val enginePodInformers: ConcurrentHashMap[KubernetesInfo, SharedIndexInformer[Pod]] =
     new ConcurrentHashMap[KubernetesInfo, SharedIndexInformer[Pod]]
+  private val engineSvcInformers: ConcurrentHashMap[KubernetesInfo, SharedIndexInformer[Service]] =
+    new ConcurrentHashMap[KubernetesInfo, SharedIndexInformer[Service]]
 
   private var submitTimeout: Long = _
   private var kyuubiConf: KyuubiConf = _
@@ -76,7 +79,8 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
     kubernetesClients.computeIfAbsent(kubernetesInfo, kInfo => buildKubernetesClient(kInfo))
   }
 
-  private var metadataManager: Option[MetadataManager] = _
+  private def metadataManager = KyuubiServer.kyuubiServer.backendService
+    .sessionManager.asInstanceOf[KyuubiSessionManager].metadataManager
 
   // Visible for testing
   private[engine] def checkKubernetesInfo(kubernetesInfo: KubernetesInfo): Unit = {
@@ -104,16 +108,18 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
           .withLabel(LABEL_KYUUBI_UNIQUE_KEY)
           .inform(new SparkEnginePodEventHandler(kubernetesInfo))
         info(s"[$kubernetesInfo] Start Kubernetes Client Informer.")
+        val engineSvcInformer = client.services()
+          .inform(new SparkEngineSvcEventHandler(kubernetesInfo))
         enginePodInformers.put(kubernetesInfo, enginePodInformer)
+        engineSvcInformers.put(kubernetesInfo, engineSvcInformer)
         client
 
       case None => throw new KyuubiException(s"Fail to build Kubernetes client for $kubernetesInfo")
     }
   }
 
-  override def initialize(conf: KyuubiConf, metadataManager: Option[MetadataManager]): Unit = {
+  override def initialize(conf: KyuubiConf): Unit = {
     kyuubiConf = conf
-    this.metadataManager = metadataManager
     info("Start initializing Kubernetes application operation.")
     submitTimeout = conf.get(KyuubiConf.ENGINE_KUBERNETES_SUBMIT_TIMEOUT)
     // Defer cleaning terminated application information
@@ -145,12 +151,10 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
       expireCleanUpTriggerCacheExecutor,
       () => {
         try {
-          Option(cleanupTerminatedAppInfoTrigger).foreach { trigger =>
-            trigger.asMap().asScala.foreach {
-              case (key, _) =>
-                // do get to trigger cache eviction
-                trigger.getIfPresent(key)
-            }
+          cleanupTerminatedAppInfoTrigger.asMap().asScala.foreach {
+            case (key, _) =>
+              // do get to trigger cache eviction
+              cleanupTerminatedAppInfoTrigger.getIfPresent(key)
           }
         } catch {
           case NonFatal(e) => error("Failed to evict clean up terminated app cache", e)
@@ -255,6 +259,11 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
     }
     enginePodInformers.clear()
 
+    engineSvcInformers.asScala.foreach { case (_, informer) =>
+      Utils.tryLogNonFatalError(informer.stop())
+    }
+    engineSvcInformers.clear()
+
     if (cleanupTerminatedAppInfoTrigger != null) {
       cleanupTerminatedAppInfoTrigger.invalidateAll()
       cleanupTerminatedAppInfoTrigger = null
@@ -264,16 +273,6 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
       Utils.tryLogNonFatalError(client.close())
     }
     kubernetesClients.clear()
-
-    if (expireCleanUpTriggerCacheExecutor != null) {
-      ThreadUtils.shutdown(expireCleanUpTriggerCacheExecutor)
-      expireCleanUpTriggerCacheExecutor = null
-    }
-
-    if (cleanupCanceledAppPodExecutor != null) {
-      ThreadUtils.shutdown(cleanupCanceledAppPodExecutor)
-      cleanupCanceledAppPodExecutor = null
-    }
   }
 
   private class SparkEnginePodEventHandler(kubernetesInfo: KubernetesInfo)
@@ -324,22 +323,85 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
     }
   }
 
+  private class SparkEngineSvcEventHandler(kubernetesInfo: KubernetesInfo)
+    extends ResourceEventHandler[Service] {
+
+    override def onAdd(svc: Service): Unit = {
+      if (isSparkEngineSvc(svc)) {
+        updateApplicationUrl(kubernetesInfo, svc)
+      }
+    }
+
+    override def onUpdate(oldSvc: Service, newSvc: Service): Unit = {
+      if (isSparkEngineSvc(newSvc)) {
+        updateApplicationUrl(kubernetesInfo, newSvc)
+      }
+    }
+
+    override def onDelete(svc: Service, deletedFinalStateUnknown: Boolean): Unit = {
+      // do nothing
+    }
+  }
+
   private def isSparkEnginePod(pod: Pod): Boolean = {
     val labels = pod.getMetadata.getLabels
     labels.containsKey(LABEL_KYUUBI_UNIQUE_KEY) && labels.containsKey(SPARK_APP_ID_LABEL)
   }
 
+  private def isSparkEngineSvc(svc: Service): Boolean = {
+    val selectors = svc.getSpec.getSelector
+    selectors.containsKey(LABEL_KYUUBI_UNIQUE_KEY) && selectors.containsKey(SPARK_APP_ID_LABEL)
+  }
+
   private def updateApplicationState(kubernetesInfo: KubernetesInfo, pod: Pod): Unit = {
     val (appState, appError) =
       toApplicationStateAndError(pod, appStateSource, appStateContainer)
     debug(s"Driver Informer changes pod: ${pod.getMetadata.getName} to state: $appState")
-    appInfoStore.put(
-      pod.getMetadata.getLabels.get(LABEL_KYUUBI_UNIQUE_KEY),
-      kubernetesInfo -> ApplicationInfo(
-        id = pod.getMetadata.getLabels.get(SPARK_APP_ID_LABEL),
-        name = pod.getMetadata.getName,
-        state = appState,
-        error = appError))
+    val kyuubiUniqueKey = pod.getMetadata.getLabels.get(LABEL_KYUUBI_UNIQUE_KEY)
+    appInfoStore.synchronized {
+      Option(appInfoStore.get(kyuubiUniqueKey)).map { case (_, appInfo) =>
+        appInfoStore.put(
+          kyuubiUniqueKey,
+          kubernetesInfo -> appInfo.copy(
+            id = pod.getMetadata.getLabels.get(SPARK_APP_ID_LABEL),
+            name = pod.getMetadata.getName,
+            state = appState,
+            error = appError))
+      }.getOrElse {
+        appInfoStore.put(
+          kyuubiUniqueKey,
+          kubernetesInfo -> ApplicationInfo(
+            id = pod.getMetadata.getLabels.get(SPARK_APP_ID_LABEL),
+            name = pod.getMetadata.getName,
+            state = appState,
+            error = appError))
+      }
+    }
+  }
+
+  private def updateApplicationUrl(kubernetesInfo: KubernetesInfo, svc: Service): Unit = {
+    svc.getSpec.getPorts.asScala.find(_.getName == SPARK_UI_PORT_NAME).map(_.getPort).map {
+      sparkUiPort =>
+        val appUrlPattern = kyuubiConf.get(KyuubiConf.KUBERNETES_SPARK_APP_URL_PATTERN)
+        val sparkAppId = svc.getSpec.getSelector.get(SPARK_APP_ID_LABEL)
+        val sparkDriverSvc = svc.getMetadata.getName
+        val kubernetesNamespace = kubernetesInfo.namespace.getOrElse("")
+        val kubernetesContext = kubernetesInfo.context.getOrElse("")
+        val appUrl = buildSparkAppUrl(
+          appUrlPattern,
+          sparkAppId,
+          sparkDriverSvc,
+          kubernetesContext,
+          kubernetesNamespace,
+          sparkUiPort)
+        debug(s"Driver Informer svc: ${svc.getMetadata.getName} app url: $appUrl")
+        val kyuubiUniqueKey = svc.getSpec.getSelector.get(LABEL_KYUUBI_UNIQUE_KEY)
+        appInfoStore.synchronized {
+          Option(appInfoStore.get(kyuubiUniqueKey)).foreach { case (_, appInfo) =>
+            appInfoStore.put(kyuubiUniqueKey, kubernetesInfo -> appInfo.copy(url = Some(appUrl)))
+          }
+        }
+    }.getOrElse(warn(s"Spark UI port not found in service ${svc.getMetadata.getName}"))
   }
 
   private def markApplicationTerminated(pod: Pod): Unit = synchronized {
@@ -400,6 +462,7 @@ object KubernetesApplicationOperation extends Logging {
   val SPARK_APP_ID_LABEL = "spark-app-selector"
   val KUBERNETES_SERVICE_HOST = "KUBERNETES_SERVICE_HOST"
   val KUBERNETES_SERVICE_PORT = "KUBERNETES_SERVICE_PORT"
+  val SPARK_UI_PORT_NAME = "spark-ui"
 
   def toLabel(tag: String): String = s"label: $LABEL_KYUUBI_UNIQUE_KEY=$tag"
 
@@ -421,20 +484,33 @@ object KubernetesApplicationOperation extends Logging {
           .find(cs => appStateContainer.equalsIgnoreCase(cs.getName))
       case KubernetesApplicationStateSource.POD => None
     }
-    val applicationState = containerStatusToBuildAppState
+
+    val podAppState = podStateToApplicationState(pod.getStatus.getPhase)
+    val containerAppState = containerStatusToBuildAppState
       .map(_.getState)
       .map(containerStateToApplicationState)
-      .getOrElse(podStateToApplicationState(pod.getStatus.getPhase))
-    val applicationError = if (ApplicationState.isFailed(applicationState)) {
-      val errorMap = containerStatusToBuildAppState.map { cs =>
-        Map("Pod" -> podName, "Container" -> appStateContainer, "ContainerStatus" -> cs)
-      }.getOrElse {
-        Map("Pod" -> podName, "PodStatus" -> pod.getStatus)
-      }
-      Some(JsonUtils.toPrettyJson(errorMap.asJava))
+
+    // When the pod app state is terminated, the container app state will be ignored
+    val applicationState = if (ApplicationState.isTerminated(podAppState)) {
+      podAppState
     } else {
-      None
+      containerAppState.getOrElse(podAppState)
     }
+    val applicationError =
+      if (ApplicationState.isFailed(applicationState)) {
+        val errorMap = containerStatusToBuildAppState.map { cs =>
+          Map(
+            "Pod" -> podName,
+            "PodStatus" -> pod.getStatus,
+            "Container" -> appStateContainer,
+            "ContainerStatus" -> cs)
+        }.getOrElse {
+          Map("Pod" -> podName, "PodStatus" -> pod.getStatus)
+        }
+        Some(JsonUtils.toPrettyJson(errorMap.asJava))
+      } else {
+        None
+      }
     applicationState -> applicationError
   }
 
@@ -466,4 +542,26 @@ object KubernetesApplicationOperation extends Logging {
         "mark the application state as UNKNOWN.")
       UNKNOWN
   }
+
+  /**
+   * Replaces all the {{SPARK_APP_ID}} occurrences with the Spark App Id,
+   * {{SPARK_DRIVER_SVC}} occurrences with the Spark Driver Service name,
+   * {{KUBERNETES_CONTEXT}} occurrences with the Kubernetes Context,
+   * {{KUBERNETES_NAMESPACE}} occurrences with the Kubernetes Namespace,
+   * and {{SPARK_UI_PORT}} occurrences with the Spark UI Port.
+   */
+  private[kyuubi] def buildSparkAppUrl(
+      sparkAppUrlPattern: String,
+      sparkAppId: String,
+      sparkDriverSvc: String,
+      kubernetesContext: String,
+      kubernetesNamespace: String,
+      sparkUiPort: Int): String = {
+    sparkAppUrlPattern
+      .replace("{{SPARK_APP_ID}}", sparkAppId)
+      .replace("{{SPARK_DRIVER_SVC}}", sparkDriverSvc)
+      .replace("{{KUBERNETES_CONTEXT}}", kubernetesContext)
+      .replace("{{KUBERNETES_NAMESPACE}}", kubernetesNamespace)
+      .replace("{{SPARK_UI_PORT}}", sparkUiPort.toString)
+  }
 }
