@@ -17,7 +17,6 @@
 
 package org.apache.kyuubi.operation
 
-import java.nio.file.{Files, Paths}
 import java.util.Locale
 import java.util.concurrent.TimeUnit
 
@@ -126,7 +125,10 @@ class BatchJobSubmission(
   }
 
   private[kyuubi] def killBatchApplication(): KillResponse = {
-    applicationManager.killApplication(builder.appMgrInfo(), batchId, Some(session.user))
+    val (killed, msg) =
+      applicationManager.killApplication(builder.appMgrInfo(), batchId, Some(session.user))
+    withOperationLog(warn(s"Kill batch response: killed: $killed, msg: $msg."))
+    (killed, msg)
   }
 
   private val applicationCheckInterval =
@@ -152,33 +154,29 @@ class BatchJobSubmission(
         engineId = appInfo.id,
         engineName = appInfo.name,
         engineUrl = appInfo.url.orNull,
-        engineState = getAppState(state, appInfo.state).toString,
+        engineState = appInfo.state.toString,
         engineError = appInfo.error,
         endTime = endTime)
       session.sessionManager.updateMetadata(metadataToUpdate)
     }
   }
 
-  private def getAppState(
-      opState: OperationState,
-      appState: ApplicationState.ApplicationState): ApplicationState.ApplicationState = {
-    if (opState == OperationState.ERROR && !ApplicationState.isTerminated(appState)) {
-      error(s"Batch $batchId state is $opState," +
-        s" but the application state is $appState and not terminated, set to UNKNOWN.")
-      ApplicationState.UNKNOWN
-    } else {
-      appState
-    }
-  }
-
   override def getOperationLog: Option[OperationLog] = Option(_operationLog)
 
   // we can not set to other state if it is canceled
   private def setStateIfNotCanceled(newState: OperationState): Unit = withLockRequired {
     if (state != CANCELED) {
       setState(newState)
-      applicationId(_applicationInfo).foreach { appId =>
-        session.getSessionEvent.foreach(_.engineId = appId)
+      _applicationInfo.foreach { app =>
+        Option(app.id).filter(_.nonEmpty).foreach { appId =>
+          session.getSessionEvent.foreach(_.engineId = appId)
+        }
+        Option(app.name).filter(_.nonEmpty).foreach { appName =>
+          session.getSessionEvent.foreach(_.engineName = appName)
+        }
+        app.url.filter(_.nonEmpty).foreach { appUrl =>
+          session.getSessionEvent.foreach(_.engineUrl = appUrl)
+        }
       }
       if (newState == RUNNING) {
         session.onEngineOpened()
@@ -242,58 +240,50 @@ class BatchJobSubmission(
   private def submitAndMonitorBatchJob(): Unit = {
     var appStatusFirstUpdated = false
     var lastStarvationCheckTime = createTime
-
-    def doUpdateApplicationInfoMetadataIfNeeded(): Unit = {
-      updateApplicationInfoMetadataIfNeeded()
-      if (!appStatusFirstUpdated) {
-        // only the ApplicationInfo with non-empty id indicates that batch is RUNNING
-        if (applicationId(_applicationInfo).isDefined) {
-          setStateIfNotCanceled(OperationState.RUNNING)
-          updateBatchMetadata()
-          appStatusFirstUpdated = true
-        } else {
-          val currentTime = System.currentTimeMillis()
-          if (currentTime - lastStarvationCheckTime > applicationStarvationTimeout) {
-            lastStarvationCheckTime = currentTime
-            warn(s"Batch[$batchId] has not started, check the Kyuubi server to ensure" +
-              s" that batch jobs can be submitted.")
-          }
-        }
-      }
-    }
-
     try {
       info(s"Submitting $batchType batch[$batchId] job:\n$builder")
       val process = builder.start
-      while (process.isAlive && !applicationFailed(_applicationInfo)) {
-        doUpdateApplicationInfoMetadataIfNeeded()
+      while (!applicationFailed(_applicationInfo) && process.isAlive) {
+        updateApplicationInfoMetadataIfNeeded()
+        if (!appStatusFirstUpdated) {
+          // only the ApplicationInfo with non-empty id indicates that batch is RUNNING
+          if (applicationId(_applicationInfo).isDefined) {
+            setStateIfNotCanceled(OperationState.RUNNING)
+            updateBatchMetadata()
+            appStatusFirstUpdated = true
+          } else {
+            val currentTime = System.currentTimeMillis()
+            if (currentTime - lastStarvationCheckTime > applicationStarvationTimeout) {
+              lastStarvationCheckTime = currentTime
+              warn(s"Batch[$batchId] has not started, check the Kyuubi server to ensure" +
+                s" that batch jobs can be submitted.")
+            }
+          }
+        }
         process.waitFor(applicationCheckInterval, TimeUnit.MILLISECONDS)
       }
 
-      if (!process.isAlive) {
-        doUpdateApplicationInfoMetadataIfNeeded()
-      }
-
       if (applicationFailed(_applicationInfo)) {
         Utils.terminateProcess(process, applicationStartupDestroyTimeout)
         throw new KyuubiException(s"Batch job failed: ${_applicationInfo}")
-      }
+      } else {
+        process.waitFor()
+        if (process.exitValue() != 0) {
+          throw new KyuubiException(s"Process exit with value ${process.exitValue()}")
+        }
 
-      if (process.waitFor() != 0) {
-        throw new KyuubiException(s"Process exit with value ${process.exitValue}")
-      }
+        while (!appStarted && applicationId(_applicationInfo).isEmpty &&
+          !applicationTerminated(_applicationInfo)) {
+          Thread.sleep(applicationCheckInterval)
+          updateApplicationInfoMetadataIfNeeded()
+        }
 
-      while (!appStarted && applicationId(_applicationInfo).isEmpty &&
-        !applicationTerminated(_applicationInfo)) {
-        Thread.sleep(applicationCheckInterval)
-        doUpdateApplicationInfoMetadataIfNeeded()
-      }
-
-      applicationId(_applicationInfo) match {
-        case Some(appId) => monitorBatchJob(appId)
-        case None if !appStarted =>
-          throw new KyuubiException(s"$batchType batch[$batchId] job failed: ${_applicationInfo}")
-        case None =>
+        applicationId(_applicationInfo) match {
+          case Some(appId) => monitorBatchJob(appId)
+          case None if !appStarted =>
+            throw new KyuubiException(s"$batchType batch[$batchId] job failed: ${_applicationInfo}")
+          case None =>
+        }
       }
     } finally {
       val waitCompletion = batchConf.get(KyuubiConf.SESSION_ENGINE_STARTUP_WAIT_COMPLETION.key)
@@ -421,11 +411,7 @@ class BatchJobSubmission(
 
   private def cleanupUploadedResourceIfNeeded(): Unit = {
     if (session.isResourceUploaded) {
-      try {
-        Files.deleteIfExists(Paths.get(resource))
-      } catch {
-        case e: Throwable => error(s"Error deleting the uploaded resource: $resource", e)
-      }
+      Utils.deleteDirectoryRecursively(session.resourceUploadFolderPath.toFile)
     }
   }
 }
