@@ -206,7 +206,7 @@ abstract class KyuubiOperation(session: Session) extends AbstractOperation(sessi
 
   protected def eventEnabled: Boolean = false
 
-  if (eventEnabled) EventBus.post(KyuubiOperationEvent(this))
+  if (eventEnabled) EventBus.post(getOperationEvent)
 
   override def setState(newState: OperationState): Unit = {
     MetricsSystem.tracing { ms =>
@@ -217,6 +217,26 @@ abstract class KyuubiOperation(session: Session) extends AbstractOperation(sessi
       ms.markMeter(MetricRegistry.name(OPERATION_STATE, newState.toString.toLowerCase))
     }
     super.setState(newState)
-    if (eventEnabled) EventBus.post(KyuubiOperationEvent(this))
+    if (eventEnabled) EventBus.post(getOperationEvent)
+  }
+
+  def getOperationEvent: KyuubiOperationEvent = {
+    val kyuubiSession = session.asInstanceOf[KyuubiSession]
+    KyuubiOperationEvent(
+      statementId,
+      Option(remoteOpHandle()).map(OperationHandle(_).identifier.toString).orNull,
+      statement,
+      shouldRunAsync,
+      state.name(),
+      lastAccessTime,
+      createTime,
+      startTime,
+      completedTime,
+      Option(operationException),
+      kyuubiSession.handle.identifier.toString,
+      kyuubiSession.user,
+      kyuubiSession.sessionType.toString,
+      kyuubiSession.connectionUrl,
+      metrics)
   }
 }
