@@ -118,6 +118,8 @@ class KyuubiSessionImpl(
     // we should call super.open before running launch engine operation
     super.open()
 
+    sessionManager.tempFileService.addPathToExpiration(operationalLogRootDir.get)
+
     runOperation(launchEngineOp)
   }
 
@@ -165,7 +167,6 @@ class KyuubiSessionImpl(
                 warn(s"Error on de-registering engine [${engine.engineSpace} $host:$port]", e)
             }
 
-          var engineClient: KyuubiSyncThriftClient = null
           try {
             val passwd =
               if (sessionManager.getConf.get(ENGINE_SECURITY_ENABLED)) {
@@ -173,11 +174,9 @@ class KyuubiSessionImpl(
               } else {
                 Option(password).filter(_.nonEmpty).getOrElse("anonymous")
               }
-            engineClient =
-              KyuubiSyncThriftClient.createClient(user, passwd, host, port, sessionConf)
+            _client = KyuubiSyncThriftClient.createClient(user, passwd, host, port, sessionConf)
             _engineSessionHandle =
-              engineClient.openSession(protocol, user, passwd, openEngineSessionConf)
-            _client = engineClient
+              _client.openSession(protocol, user, passwd, openEngineSessionConf)
             logSessionInfo(s"Connected to engine [$host:$port]/[${client.engineId.getOrElse("")}]" +
               s" with ${_engineSessionHandle}]")
             shouldRetry = false
@@ -208,9 +207,9 @@ class KyuubiSessionImpl(
               throw e
           } finally {
             attempt += 1
-            if (shouldRetry && engineClient != null) {
+            if (shouldRetry && _client != null) {
               try {
-                engineClient.closeSession()
+                _client.closeSession()
               } catch {
                 case e: Throwable =>
                   warn(
@@ -224,6 +223,8 @@ class KyuubiSessionImpl(
         sessionEvent.openedTime = System.currentTimeMillis()
         sessionEvent.remoteSessionId = _engineSessionHandle.identifier.toString
         _client.engineId.foreach(e => sessionEvent.engineId = e)
+        _client.engineName.foreach(e => sessionEvent.engineName = e)
+        _client.engineUrl.foreach(e => sessionEvent.engineUrl = e)
         EventBus.post(sessionEvent)
       }
     }
@@ -234,7 +235,7 @@ class KyuubiSessionImpl(
         waitForEngineLaunched()
       } catch {
         case t: Throwable =>
-          sessionManager.operationManager.closeOperation(operation.getHandle)
+          operation.close()
           throw t
       }
       sessionEvent.totalOperations += 1
